/*
	Program	: DB Util
	Class   : dbf
	Version : 0.1 (initial release)
	Author  : Costin Grigoras <costing@mymail.ro>
*/

import java.sql.*;
import java.util.*;

public class dbf
{
    private   Statement    batchStatement   = null;
    private   String       sCurrentSQLQuery = "";
    private   boolean      first            = true;
    private   boolean	   isUpdate         = false;
    protected ResultSet    rsRezultat       = null;
    public    int          iCurrentPos      = 0;
    
    private   DBConnection dbc;
    
    static    List         lConnections;
    
    static{
	lConnections = new LinkedList();
    };
    
    public final static int PROVIDER_MYSQL   =1;
    public final static int PROVIDER_POSTGRES=2;
    public final static int PROVIDER_MSSQL   =3;
    
    private class DBConnection{
	private  int        iProvider  = 0;
	private  String     sDatabase  = "";
	private  Connection connex     = null;
	private  boolean    bConnected = false;
	
	public   Object     lock;
	
	private String sHost = "";
	private String sUser = "";
	private String sPass = "";
	
	private DBConnection(){};
	
	public DBConnection(int _iProvider, String _sHost, String _sUser, String _sPass, String _sDatabase){
	    lock = new String("");
	    connect(_iProvider, _sHost, _sUser, _sPass, _sDatabase);
	}
	
	public boolean reconnect(){
	    System.err.println("dbf.DBConnection : reconnect requested");
	    return connect(iProvider, sHost, sUser, sPass, sDatabase);
	}
	
	public boolean connect(int _iProvider, String _sHost, String _sUser, String _sPass, String _sDatabase){
	    iProvider= _iProvider;
	    sDatabase= new String(_sDatabase);
	    sHost    = new String(_sHost);
	    sUser    = new String(_sUser);
	    sPass    = new String(_sPass);
	    
	    connex = null;
	    
	    switch(iProvider){
		case PROVIDER_MYSQL: 
		    try{
			Class.forName("org.gjt.mm.mysql.Driver");
        		connex = DriverManager.getConnection("jdbc:mysql://"+_sHost+"/"+sDatabase+"/", _sUser, _sPass);
			bConnected = true;
			return true;
		    }
		    catch (Exception e){
		        System.err.println("dbf.DBConnection : error connecting to mysql database "+sDatabase+"@"+_sHost+" ! " + e.getMessage());
			bConnected = false;
			return false;
		    }
		    
		case PROVIDER_POSTGRES:
		    try{
			System.setProperty("PGDATESTYLE", "ISO");
		    }
		    catch (Exception e){
		    }
		
		    try{
        		Class.forName("org.postgresql.Driver");
        		connex = DriverManager.getConnection("jdbc:postgresql://"+_sHost+"/"+sDatabase+"/", _sUser, _sPass);
			bConnected = true;
			return true;
		    }
		    catch (Exception e){
			System.err.println("dbf.DBConnection : error connecting to postgres database "+sDatabase+"@"+_sHost+" ! " + e.getMessage());
			bConnected = false;
			return false;
		    }

		case PROVIDER_MSSQL:
		    try{
        		Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver");
        		connex = DriverManager.getConnection("jdbc:microsoft:sqlserver://"+_sHost+";User="+_sUser+";Password="+_sPass+";DatabaseName="+sDatabase);
			bConnected = true;
			return true;
		    }
		    catch (Exception e){
			System.err.println("dbf.DBConnection : error connecting to mssql database "+sDatabase+"@"+_sHost+" ! "+ e.getMessage());
			bConnected = false;
			return false;
		    }
		    
		default:
		    bConnected = false;
		    return false;
	    }
	}
	
	public boolean connected(){
	    return bConnected;
	};
	
	public Connection getConnection(){
	    return connex;
	}
	
	public int getProvider(){
	    return iProvider;
	}
	
	public String getDatabase(){
	    return sDatabase;
	}
    }
    
    public dbf(int iProvider, String sHost, String sUser, String sPass, String sDatabase){
	Iterator it = lConnections.iterator();

	dbc = null;

	while (it.hasNext()){
	    DBConnection dbcTemp = (DBConnection) it.next();
	    
	    if ( (dbcTemp.getProvider() == iProvider) && (dbcTemp.getDatabase().toLowerCase().equals(sDatabase.toLowerCase())) ){
		dbc = dbcTemp;
		break;
	    }
	}
	
	if (dbc==null){
	    DBConnection dbcTemp = new DBConnection(iProvider, sHost, sUser, sPass, sDatabase);
	    
	    if (dbcTemp.connected()){
		lConnections.add(dbcTemp);
		dbc = dbcTemp;
		if (iProvider == PROVIDER_POSTGRES){
		    update("SET DateStyle TO 'ISO';");
		}
	    }
	    else{
		System.err.println("dbf : cannot open the new connection!");
	    }
	}
    }
    
    public void reconnect(){
	try{
	    dbc.reconnect();
	}
	catch (Exception e){
	}
    }
    
    public boolean update(String sSQLQuery){
	return query(sSQLQuery, true);
    }
    
    public boolean query(String sSQLQuery){
	return query(sSQLQuery, false);
    }
    
    public boolean query(String sSQLQuery, boolean bIsUpdate){
	rsRezultat = null;
	sCurrentSQLQuery = "";
	
	isUpdate = bIsUpdate;
	
	if ((dbc==null) || (!dbc.connected())) {
	    dbc.reconnect();
	    if ((dbc==null) || (!dbc.connected())){
		return false;
	    }
	}
	
        try {
	  synchronized (dbc.lock){
            Statement stat = dbc.getConnection().createStatement();
	    
	    if (!isUpdate) rsRezultat = stat.executeQuery(sSQLQuery);
	    else	   stat.executeUpdate(sSQLQuery);
	    
	    sCurrentSQLQuery = new String(sSQLQuery);
	    
	    if (!isUpdate){
		first = true;
	        try{
	    	    if (!rsRezultat.next())
			first = false;
		}
		catch (Exception e){
		    first = false;
		}
	    }
	    else
		first = false;
	  }
	}
        catch (Exception e) {
	    String s = e.getMessage();
	    if ( 
	    	(s!=null) && 
	        (s.indexOf("Cannot insert a duplicate key")<0) && 
		(s.indexOf("[Microsoft][SQLServer 2000 Driver for JDBC]No ResultSet set was produced")<0)
	    ){
    		System.err.println("db.dbf.query : error at '"+ sSQLQuery +"' with message "+ s);
		dbc.reconnect();
	    }
            return false;
        }
	
        iCurrentPos = 0;
        return true;
    }
    
    public ResultSetMetaData getMetaData(){
	if (rsRezultat != null)
	    try{
		return rsRezultat.getMetaData();
	    }
	    catch (Exception e){
	    }
	
	return null;
    }
    
    
    public boolean moveNext(){
	if (isUpdate) return false;
    
	if (first){
	    first = false;
	    return true;
	}
    
	if (rsRezultat!=null){
	    try{
		if (!rsRezultat.next()) return false;
		iCurrentPos ++;
		return true;
	    }
	    catch (Exception e){
		return false;
	    }
	}
	else
	    return false;
    }
    
    public boolean requery(){
	if (rsRezultat!=null && sCurrentSQLQuery.length()>0)
	    return query(sCurrentSQLQuery, isUpdate);
	else
	    return false;
    }

    public String gets(String sColumnName){
	return gets(sColumnName, "");
    }
    
    public String gets(String sColumnName, String sDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return sDefault;
	try{
	    String sTemp = rsRezultat.getString(sColumnName);
	    if (sTemp==null) sTemp = sDefault;
    	    return sTemp.trim();
	}
	catch (Exception e){
	    return sDefault;
	}
    }
    
    public int geti(String sColumnName){
	return geti(sColumnName, 0);
    }
    
    public int geti(String sColumnName, int iDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return iDefault;
	try{
	    int iTemp = rsRezultat.getInt(sColumnName);
	    if (rsRezultat.wasNull()) return iDefault;
	    return iTemp;
	}
	catch (Exception e){
	    return iDefault;
	}
    }

    public long getl(String sColumnName){
	return getl(sColumnName, 0);
    }
    
    public long getl(String sColumnName, long lDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return lDefault;
	try{
	    long lTemp = rsRezultat.getLong(sColumnName);
	    if (rsRezultat.wasNull()) return lDefault;
	    return lTemp;
	}
	catch (Exception e){
	    return lDefault;
	}
    }

    public float  getf(String sColumnName){
	return getf(sColumnName, 0);
    }
    
    public float getf(String sColumnName, float fDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return fDefault;
	try{
	    float fTemp = rsRezultat.getFloat(sColumnName);
	    if (rsRezultat.wasNull()) return fDefault;
	    return fTemp;
	}
	catch (Exception e){
	    return fDefault;
	}
    }

    public String gets(int iColumn){
	return gets(iColumn, "");
    }
    
    public String gets(int iColumn, String sDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return sDefault;
	try{
	    String sTemp = rsRezultat.getString(iColumn);
	    if (sTemp==null) sTemp = sDefault;
    	    return sTemp;
	}
	catch (Exception e){
	    return sDefault;
	}
    }

    public int geti(int iColumn){
	return geti(iColumn, 0);
    }
    
    public int geti(int iColumn, int iDefault){
	if ((dbc==null) || !dbc.connected() || rsRezultat==null) return iDefault;
	try{
	    int iTemp = rsRezultat.getInt(iColumn);
	    if (rsRezultat.wasNull()) return iDefault;
	    return iTemp;
	}
	catch (Exception e){
	    return iDefault;
	}
    }
}
