/*
	Program	: DB Util
	Class   : dbutil
	Version : 0.1 (initial release)
	Author  : Costin Grigoras <costing@mymail.ro>
*/

import java.io.*;
import java.sql.*;
import java.util.*;

public class dbutil {

    static String sHost;
    static String sUser;
    static String sPass;
    static String sDatabase;
    static int    iProvider;
    static int    arg_mark[];

    private static String getArgument(String args[], String sParam){
    	for (int i=0; i<args.length; i++)
		if (args[i].startsWith("-"+sParam)){
			arg_mark[i] = 1;
		
			if (args[i].length() > 2)
				return args[i].substring(2);
			else
				if ((i<args.length-1) && (!args[i+1].startsWith("-"))){
					arg_mark[i+1] = 1;
					return args[i+1];
				}
				else
					return null;
		}
		
	return null;
    }

    private static void parseCmdLine(String args[]){
    	arg_mark = new int[args.length];
    
    	sHost = "127.0.0.1";
	sUser = "";
	sPass = "";
	sDatabase = "";
    
    	String s = null;
    
	iProvider = dbf.PROVIDER_MSSQL;
	
	s = getArgument(args, "t");
	
	if (s!=null){
		s = s.toLowerCase().trim();
		
		if (s.equals("mysql"))
			iProvider = dbf.PROVIDER_MYSQL;
		else if (s.equals("pgsql") || s.equals("postgres")) 
			iProvider = dbf.PROVIDER_POSTGRES;
		else if (s.equals("mssql")) 
			iProvider = dbf.PROVIDER_MSSQL;
		else
			System.err.println("WARNING : Unknown database type : "+s);
	}
	
	switch (iProvider){
		case dbf.PROVIDER_MSSQL:
			sUser = "sa";
			sDatabase = "master";
			break;
			
		case dbf.PROVIDER_MYSQL:
			sUser = "root";
			sDatabase = "mysql";
			break;
		
		case dbf.PROVIDER_POSTGRES:
			sUser = "postgres";
			sDatabase = "template1";
			break;
	}
	
	s = getArgument(args, "u");
	if (s!=null && s.length()>0)
		sUser = s;

	s = getArgument(args, "p");
	if (s!=null && s.length()>0)
		sPass = s;
		
	s = getArgument(args, "h");
	if (s!=null && s.length()>0)
		sHost = s;
		
	s = getArgument(args, "d");
	if (s!=null && s.length()>0)
		sDatabase = s;
    }

    public static void main(String args[]){

	parseCmdLine(args);

	InputStream is = null;
	
	int iFile = -1;

	for (int ii=0; ii<arg_mark.length; ii++)	
		if (arg_mark[ii] == 0){
			iFile = ii;
			break;
		}

	if ( iFile < 0 )
	    is = System.in;
	else
	    try{
		is = new FileInputStream(args[iFile]);
	    }
	    catch (Exception e){
		System.out.println("Cannot find file : "+args[iFile]);
	    }
	    
	if (is==null){
	    System.out.println("Exiting ... ");
	    return;
	}
	
	BufferedReader br = new BufferedReader(new InputStreamReader(is));
	    
	StringBuffer sb = new StringBuffer(10000);
	
	dbf db = new dbf(iProvider, sHost, sUser, sPass, sDatabase);
	
	String sLine = "";
	
	int count = 0;
	
	int lim=200;
	
	boolean bFirst = true;
	
	while (true){
		// wait for a complete query
		
		int iP = 0;
		boolean bG = false;
		boolean bE = false;
		boolean exit = false;
		
		bFirst = true;
		
		sb = new StringBuffer(10000);
		
		do{
			String ssb;
			
			ssb = sb.toString();
			
			while (ssb.length()>0 && ssb.startsWith("\n"))
				ssb = ssb.substring( "\n".length() );
				
			if (ssb.startsWith("\\c ") || ssb.toLowerCase().startsWith("use "))
				break;
		
			if (iFile < 0){
				if (bG) System.out.print("str> ");
				else if (iP>0) System.out.print(iP+" brc> ");
				else if (iP<0) System.out.print("err> ");
				else if (sb.toString().length()<=0) System.out.print(sDatabase+"# ");
				else System.out.print("cnt> ");
			
				System.out.flush();
			}
		
			try{
				if (bFirst && (sLine!=null) && (sLine.length()>0)){
				}
				else{
				     	sLine = br.readLine();
				}
					
				if (sLine!=null)
					sLine = sLine.trim();
			}
			catch (Exception e){
				break;
			}
			
			bFirst = false;
			
			if (sLine==null){
				System.err.println("bye bye");
				return;
			}
			
			sLine = "\n"+sLine;
			
			for (int ic=0; ic<sLine.length(); ic++){
				char c = sLine.charAt(ic);
				
				sb.append(c);
				
				if (bG){
					if (c=='\'' && (!bE) && ((ic==sLine.length()-1) || sLine.charAt(ic+1)!='\'')) bG = false;
					if (c=='\\' || (c=='\'' && (ic<sLine.length()-1) && (sLine.charAt(ic+1)=='\''))) bE = !bE;
					else bE = false;
				}
				else{
					if (c=='(') iP++;
					if (c==')') iP--;
					if (c=='\'') bG = true;
					if (c==';'){
						if (iP != 0)
							System.err.println("Hmm, numarul de paranteze e aiurea ...");
						
						if (ic<=sLine.length()-1)
							sLine = sLine.substring(ic+1);
						else
							sLine = "";
							
						exit = true;
						
						break;
					}
				}
			}
		}
		while (!exit);
		
		String sQuery = sb.toString();
		
		while (sQuery.length()>0 && sQuery.startsWith("\n"))
			sQuery = sQuery.substring( "\n".length() );
		
		if (sQuery.toLowerCase().startsWith("use ") || sQuery.startsWith("\\c ")){
			try{
				StringTokenizer st = new StringTokenizer(sQuery, " \r\n\t;");
				st.nextToken();
				sDatabase = st.nextToken();
				db = new dbf(iProvider, sHost, sUser, sPass, sDatabase);
				System.out.println("CONNECT - "+sDatabase);
				Thread.sleep(1000);
			}
			catch (Exception e){
				System.err.println("Error parsing the new database connection!");
			}
			
			sb = new StringBuffer(10000);
			sLine = "";
			
			continue;
		}	
		
		boolean bOK = db.query(sQuery);

		ResultSetMetaData rsmd = db.getMetaData();
	
		int i, j;
		
		boolean bSomething;
		
		bSomething = false;
		
		StringBuffer sbl;
	
		try{
	 		if (rsmd != null){
				count=rsmd.getColumnCount();
	        		int total=0;
				
				sbl = new StringBuffer(5000);
	    
	        		for (i=1; i<=count; i++){
	    	    			String name = rsmd.getColumnName(i);
	    	    			int    w    = rsmd.getColumnDisplaySize(i);
		    
		    			if (w > lim)
						w = lim;
	    	
		    			sbl.append(name); total += name.length();
		
		    			for (j=0; j<w-name.length(); j++){
						sbl.append(' ');
						total ++;
		    			}
		    
		    			if (i<count){
		        			sbl.append(" | ");
						total += 3;
		    			}
				}
				
				if (total>0){
					System.out.println(sbl.toString());
					bSomething = true;
				}
	    	
				for (i=0; i<total; i++)
		    			System.out.print("-");
		
				if (total>0)
					System.out.println("");
				
	    		}
			else
				System.out.println("DONE");
		}
		catch (Exception e){
	    		System.err.println("Exceptie: "+e+" ("+e.getMessage()+")");
		}
		
		while (bSomething && db.moveNext()){
			sbl = new StringBuffer(5000);
		
	    		for (i=1; i<=count; i++){
				String name = "";
				int w=0;
	    
				try{
	    	    			name = rsmd.getColumnName(i);
	    	    			w    = rsmd.getColumnDisplaySize(i);
				}	
				catch (Exception e){
		    			System.err.println("!?!?!?! "+e+" ("+e.getMessage()+")");
				}
		
				if (w < name.length()) w = name.length();
			
				if (w > lim) w = lim;
		
				int l = db.gets(name).length();
		
				sbl.append(db.gets(name));
		
				for (j=0; j<w-l; j++)
		    			sbl.append(' ');
		
				if (i<count)
		    			sbl.append(" | ");
	    		}
	    
	    		System.out.println(sbl.toString());
		}
		
		if (bSomething)
			System.out.println("\n");
    
    	}

    }

}
